@@ -3,6 +3,7 @@
 """This test unit checks node attributes that are persistent (AttributeSet)."""
 
 from __future__ import print_function
+from __future__ import absolute_import
 import sys
 import datetime
 from distutils.version import LooseVersion
@@ -16,8 +17,9 @@ from tables import (IsDescription, Int32Atom, StringCol, IntCol, Int16Col,
 from tables.exceptions import DataTypeWarning
 from tables.parameters import NODE_CACHE_SLOTS
 from tables.tests import common
-from tables.tests.common import unittest
+from tables.tests.common import unittest, test_filename
 from tables.tests.common import PyTablesTestCase as TestCase
+from six.moves import range
 
 
 class Record(IsDescription):
@@ -69,6 +71,91 @@ class CreateTestCase(common.TempFileMixin, TestCase):
         self.assertEqual(self.h5file.get_node_attr(self.root.anarray, 'attr1'),
                          "n" * attrlength)
 
+    def reopen(self):
+        # Reopen
+        if self.close:
+            if common.verbose:
+                print("(closing file version)")
+            self._reopen(mode='r+', node_cache_slots=self.node_cache_slots)
+            self.root = self.h5file.root
+
+    def check_missing(self,name):
+        self.reopen()
+        self.assertFalse(name in self.root.agroup._v_attrs)
+        self.assertFalse(name in self.root.atable.attrs)
+        self.assertFalse(name in self.root.anarray.attrs)
+
+
+    def check_name(self, name, val = ''):
+        """Check validity of attribute name filtering"""
+        self.check_missing(name)
+        # Using File methods
+        self.h5file.set_node_attr(self.root.agroup, name, val)
+        self.h5file.set_node_attr(self.root.atable, name, val)
+        self.h5file.set_node_attr(self.root.anarray, name, val)
+         # Check File methods
+        self.reopen()
+        self.assertEqual(self.h5file.get_node_attr(self.root.agroup, name),val)
+        self.assertEqual(self.h5file.get_node_attr(self.root.atable, name),val)
+        self.assertEqual(self.h5file.get_node_attr(self.root.anarray, name),val)
+        # Remove, file methods
+        self.h5file.del_node_attr(self.root.agroup, name)
+        self.h5file.del_node_attr(self.root.atable, name)
+        self.h5file.del_node_attr(self.root.anarray, name)
+        self.check_missing(name)
+
+        # Using Node methods
+        self.root.agroup._f_setattr(name, val)
+        self.root.atable.set_attr(name, val)
+        self.root.anarray.set_attr(name, val)
+        # Check Node methods
+        self.reopen()
+        self.assertEqual(self.root.agroup._f_getattr(name), val)
+        self.assertEqual(self.root.atable.get_attr(name), val)
+        self.assertEqual(self.root.anarray.get_attr(name), val)
+        self.root.agroup._f_delattr(name)
+        self.root.atable.del_attr(name)
+        self.root.anarray.del_attr(name)
+        self.check_missing(name)
+
+        # Using AttributeSet methods
+        setattr(self.root.agroup._v_attrs, name, val)
+        setattr(self.root.atable.attrs, name, val)
+        setattr(self.root.anarray.attrs, name, val)
+        # Check AttributeSet methods
+        self.reopen()
+        self.assertEqual(getattr(self.root.agroup._v_attrs, name), val)
+        self.assertEqual(getattr(self.root.atable.attrs, name), val)
+        self.assertEqual(getattr(self.root.anarray.attrs, name), val)
+        delattr(self.root.agroup._v_attrs,name)
+        delattr(self.root.atable.attrs, name)
+        delattr(self.root.anarray.attrs, name)
+        self.check_missing(name)
+
+        # Using dict []
+        self.root.agroup._v_attrs[name]=val
+        self.root.atable.attrs[name]=val
+        self.root.anarray.attrs[name]=val
+        # Check dict []
+        self.reopen()
+        self.assertEqual(self.root.agroup._v_attrs[name], val)
+        self.assertEqual(self.root.atable.attrs[name], val)
+        self.assertEqual(self.root.anarray.attrs[name], val)
+        del self.root.agroup._v_attrs[name]
+        del self.root.atable.attrs[name]
+        del self.root.anarray.attrs[name]
+        self.check_missing(name)
+
+    def test01a_setAttributes(self):
+        """Checking attribute names validity"""
+        self.check_name('a')
+        self.check_name('a:b')
+        self.check_name('/a/b')
+        self.check_name('.')
+        self.assertRaises(ValueError, self.check_name, '')
+        self.assertRaises(ValueError, self.check_name, '__members__')
+        self.assertRaises(TypeError, self.check_name, 0)
+
     def test02_setAttributes(self):
         """Checking setting large string attributes (Node methods)"""
 
@@ -515,6 +602,47 @@ class CreateTestCase(common.TempFileMixin, TestCase):
         assert_array_equal(self.array.attrs['b'], data.T)
         assert_array_equal(self.array.attrs['c'], data.T)  # AssertionError!
 
+    @unittest.skipIf(sys.version_info[0] < 3, "Special method `__dir__()` introduced in Python-3.")
+    def test12_dir(self):
+        """Checking AttributeSet.__dir__"""
+
+        if common.verbose:
+            print('\n', '-=' * 30)
+            print("Running %s.test12_dir..." % self.__class__.__name__)
+
+        attrset = self.group._v_attrs
+
+        user_attr = 'good_attr'
+        sys_attr = 'BETTER_ATTR'
+        bad_user = '5bad'
+        bad_sys = 'SYS%'
+        for a in [user_attr, sys_attr, bad_user, bad_sys]:
+            attrset[a] = 1
+
+        completions = dir(attrset)
+
+        ## Check some regular attributes.
+        #
+        self.assertIn('__class__', completions)
+        self.assertIn('_f_copy', completions)
+        self.assertEqual(completions.count('_f_copy'), 1)
+
+        ## Check SYS attrs.
+        #
+        self.assertNotIn(bad_sys, completions)
+        self.assertIn(sys_attr, completions)
+        self.assertEqual(completions.count(sys_attr), 1)
+
+        ## Check USER attrs.
+        #
+        self.assertIn(user_attr, completions)
+        self.assertNotIn(bad_user, completions)
+        self.assertEqual(completions.count(user_attr), 1)
+
+        # Now check all for no duplicates.
+        self.assertSequenceEqual(sorted(set(completions)),
+                                 sorted(completions))
+
 
 class NotCloseCreate(CreateTestCase):
     close = False
@@ -696,8 +824,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (scalar, NumPy case)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype, numpy.array(1, dtype=dtype))
@@ -723,8 +851,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (unidimensional NumPy case)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype, numpy.array([1, 2], dtype=dtype))
@@ -748,8 +876,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (unidimensional, non-contiguous)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             arr = numpy.array([1, 2, 3, 4], dtype=dtype)[::2]
@@ -774,8 +902,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (bidimensional NumPy case)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -827,7 +955,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test02b_setFloatAttributes(self):
         """Checking setting Float attributes (scalar, NumPy case)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -854,7 +982,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test02c_setFloatAttributes(self):
         """Checking setting Float attributes (unidimensional NumPy case)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -881,7 +1009,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Float attributes (unidimensional,
         non-contiguous)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             arr = numpy.array([1.1, 2.1, 3.1, 4.1], dtype=dtype)[1::2]
@@ -907,7 +1035,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test02e_setFloatAttributes(self):
         """Checking setting Int attributes (bidimensional NumPy case)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -1140,7 +1268,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test05c_setComplexAttributes(self):
         """Checking setting Complex attributes (unidimensional NumPy case)"""
 
-        checktypes = ['Complex32', 'Complex64']
+        checktypes = ['complex64', 'complex128']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -1166,7 +1294,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test05d_setComplexAttributes(self):
         """Checking setting Int attributes (bidimensional NumPy case)"""
 
-        checktypes = ['Complex32', 'Complex64']
+        checktypes = ['complex64', 'complex128']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -1497,7 +1625,7 @@ class NoSysAttrsClose(NoSysAttrsTestCase):
 
 
 class CompatibilityTestCase(common.TestFileMixin, TestCase):
-    h5fname = TestCase._testFilename('issue_368.h5')
+    h5fname = test_filename('issue_368.h5')
 
     @unittest.skipIf(LooseVersion(numpy.__version__) < '1.9.0',
                      'requires numpy >= 1.9')
@@ -1521,7 +1649,7 @@ class CompatibilityTestCase(common.TestFileMixin, TestCase):
 
 
 class PicklePy2UnpicklePy3TestCase(common.TestFileMixin, TestCase):
-    h5fname = TestCase._testFilename('issue_560.h5')
+    h5fname = test_filename('issue_560.h5')
 
     @unittest.skipIf(sys.version_info[0] == 3 and sys.version_info[1] < 4,
                      'bug not fixed on python3<=3.3.')
@@ -1533,12 +1661,18 @@ class PicklePy2UnpicklePy3TestCase(common.TestFileMixin, TestCase):
         # a UnicodeDecodeError when unpickling on python 3.
         # Python 3.4 adds encoding='bytes' to fix this
         # http://bugs.python.org/issue6784
-        # This is not limited to datetime.datetime()
+        # Objects pickled in the testfile have non-ascii chars in the
+        # picklestring and will throw UnicodeDecodeError when unpickled
+        # on python 3.
 
+        # datetime will be unpickled with encoding='bytes'
         self.assertIsInstance(
             self.h5file.get_node_attr('/', 'py2_pickled_datetime'),
             datetime.datetime)
-
+        # dict will be unpickled with encoding='latin1'
+        d = self.h5file.get_node_attr('/', 'py2_pickled_dict')
+        self.assertIsInstance(d, dict)
+        self.assertEqual(d['s'], u'just a string')
 
 class SegFaultPythonTestCase(common.TempFileMixin, TestCase):
 
@@ -1584,7 +1718,7 @@ class EmbeddedNullsTestCase(common.TempFileMixin, TestCase):
 class VlenStrAttrTestCase(TestCase):
     def setUp(self):
         super(VlenStrAttrTestCase, self).setUp()
-        self.h5fname = self._testFilename('vlstr_attr.h5')
+        self.h5fname = test_filename('vlstr_attr.h5')
         self.h5file = tables.open_file(self.h5fname)
 
     def tearDown(self):
@@ -1621,7 +1755,7 @@ class VlenStrAttrTestCase(TestCase):
 
 
 class UnsupportedAttrTypeTestCase(common.TestFileMixin, TestCase):
-    h5fname = TestCase._testFilename('attr-u16.h5')
+    h5fname = test_filename('attr-u16.h5')
 
     def test00_unsupportedType(self):
         """Checking file with unsupported type."""
